@echo off
REM
REM   DEMO.BAT demonstrates the PICHEX toolkit.  Make sure the toolkit
REM   programs are in PATH and FIRST.HEX is in the same directory
REM   as DEMO.BAT
REM
REM   17-Jul-1996  V-0.1
REM
REM   Copyright (C) David Tait 1996
REM
CLS
echo.
echo                PICHEX Toolkit Demonstration
echo.
echo The PICHEX toolkit is designed to manipulate Microchip hex files.
echo It can be compiled in library form for use in your own programs
echo or built as individual programs.  This demo will illustrate the
echo standalone versions of the toolkit functions.
echo.
echo The toolkit is intended for use with PIC16C84 files in particular.
echo A PIC16C84 has four programmable areas: program memory, data memory,
echo 4 ID words and a configuration word; a Microchip hex file stores all
echo or part of the information needed to program these areas.  The files
echo use a encoding scheme (defined many years ago by Intel) that consists
echo of structured sequences of hex digits.
echo.
echo A single hex digit encodes a 4-bit binary number or "nibble":
echo '0' through '9' is used for 0000 to 1001 and 'A' through 'F' for
echo 1010 to 1111.  An 8-bit byte can be represented as two hex digits
echo the first giving the most significant nibble and the second giving
echo the least significant nibble.
echo.
echo A short tutorial on Microchip hex file format follows.
echo.
PAUSE
CLS
echo.
echo Here is the first line (or record) from a typical hex file:
echo.
echo     :10000000863084008001860181308400640087305E
echo.
echo This record has the following structure:
echo.
echo     :BBAAAATTHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHCC
echo.
echo BB is a one byte (a pair of hex digits is one byte) count of the
echo number of data elements in the record; it is 10 (16 decimal) here.
echo AAAA is the starting address of the record (0000 here).  TT is 00
echo on all records except the final end-of-file record.  H represents
echo a hex digit in the data field.  CC is a checksum byte included for
echo robustness (5E here).  The algorithm for calculating the checksum
echo is quite straightforward and is described next.
echo.
echo.
PAUSE
CLS
echo.
echo The checksum is the two's complement of the sum of all bytes in the
echo record (including the count, address and TT bytes).  The sum
echo is calculated using 8-bit arithmetic (i.e. modulo 256 decimal).  For
echo the example record the sum is (00 bytes don't affect the sum):
echo.
echo   10 + 86 + 30 + 84 + 80 + 01 + 86 + 01 + 81 + 30 + 84 + 64 + 87 + 30
echo.
echo.  = A2
echo.
echo The checksum is therefore 5E (because 5E = -A2 using 8-bit arithmetic).
echo.
echo.
PAUSE
CLS
echo.
echo There are two styles of Microchip hex file in common use.  If
echo the data elements are byte-oriented then BB gives a data byte count,
echo addresses increment by one for each byte, and the data elements are
echo bytes given by BB consecutive pairs of hex digits.  This format is
echo called INHX8M (8-bit merged format); the previous example was in
echo INHX8M format.  INHX8M is the default hex style used by Microchip's
echo MPASM assembler.  The MPALC assembler used a different default format
echo and although MPALC is obsolete it is still used and means that
echo this second file format is also found.
echo.
echo.
PAUSE
CLS
echo.
echo The second style of hex file is called INHX16; in this format BB gives
echo a word count (a word is two bytes), addresses increment by one for
echo each word, and data elements are words given by BB consecutive
echo groups of four hex digits.  Here is the INHX8M record again:
echo.
echo     :10000000863084008001860181308400640087305E
echo.
echo and the equivalent in INHX16 format looks like this:
echo.
echo     :080000003086008401800186308100840064308766
echo.
echo You can see that BB is different in each (10 in INHX8M, 08 in INHX16).
echo The order of consecutive data byte pairs has been reversed (e.g.
echo the first two data bytes in the INHX8M style are 8630 but in INHX16
echo they are 3086).  PIC16C84 memory is 14-bits wide and needs two
echo bytes or one word to define the contents each location.  INHX16 has
echo the bytes in the "correct" order - most significant byte followed
echo by least significant byte.  The checksum is calculated the same
echo way in both styles but because it includes the data count it
echo is different for each.
echo.
PAUSE
CLS
echo.
echo Looking at consecutive records in each style shows another
echo difference.
echo.
echo   INHX8M:
echo.
echo     :10000000863084008001860181308400640087305E
echo     :100010008000FF3086001020003086001020092864
echo.
echo   INHX16:
echo.
echo     :080000003086008401800186308100840064308766
echo     :08000800008030FF00862010300000862010280974
echo.
echo The starting address of the second record is 0008 in INHX16
echo (because WORDS 0-7 were in the first record) but 0010 (16 decimal)
echo in INHX8M (because BYTES 0-16 were in the first record).  Both
echo files say the PIC16C84 should have 3086 at address 0000
echo and 0080 at address 0008 (the PIC program memory stores 14-bits
echo per address and therefore needs one word or two bytes to encode
echo the contents).
echo.
PAUSE
CLS
echo The PIC16C84 has 1024 14-bit words of program memory with
echo addresses in the range 0000-03FF and 64 bytes of data memory
echo logically addressed in the range 2100-213F.  It also has
echo 4 ID words that are logically addressed 2000-2003 and a
echo configuration word at logical address 2007.  Here are typical
echo ID, configuration and data records in both styles:
echo.
echo ID @ 2000:      INHX8M  :084000000A000B000C000D008A
echo                 INHX16  :04200000000A000B000C000DAE
echo.
echo config @ 2007:  INHX8M: :02400E00FD3F74
echo                 INHX16: :012007003FFD9C
echo.
echo data @ 2100:    INHX8M: :1042000073006F006D0065002000640061007400A1
echo                 INHX16: :082100000073006F006D00650020006400610074CA
echo.
echo Notice that the addresses in the INHX8M records are double the
echo real values (as used in INHX16), and the most-significant bytes of
echo the data words are 00 in both styles because data memory is only
echo one byte wide.
echo.
echo.
PAUSE
CLS
echo.
echo The hex file may not contain information for all four areas but
echo if it does they are easy to spot just by looking at the address fields
echo of the records - the WHATHEX program (see later) makes this even
echo easier.
echo.
echo Finally, all valid hex files have an end-of-file (EOF) record.  The
echo EOF record looks the same in both INHX8M and INHX16 and is
echo distinguished from ordinary records by a TT byte of 01 rather
echo than 00:
echo.
echo     :00000001FF
echo.
echo.
PAUSE
CLS
echo.
echo Though it is quite easy to differentiate an INHX8M file from
echo an INHX16 file by eye it can be done much more efficiently
echo by using a simple program.  The toolkit program WHATHEX not only
echo does this but also verifies a file contains only legal hex records
echo and identifies the presence or otherwise of each of the four
echo programmable areas.  WHATHEX also computes overall checksums two
echo different ways.
echo.
echo.
PAUSE
CLS
echo.
echo Here is the contents of a file called FIRST.HEX (*):
echo.
TYPE FIRST.HEX
echo.
echo Legitimate  questions might include: Is this a legal hex file?
echo Is it INHX8M or INHX16?  What are the ID words?  What
echo configuration does it specify?  Does it have any data records?
echo.
echo (*) It's common to use .HEX as the file extension but .OBJ, .H16,
echo     .H08 and .EEP (for data-only files) are also common.
echo.
echo.
PAUSE
CLS
echo.
echo Now here's what WHATHEX says about FIRST.HEX:
echo.
WHATHEX FIRST.HEX
echo.
echo This tells us that FIRST.HEX is an INHX8M file containing program,
echo ID, config and data areas.  The first number after "Checksum:" is
echo a checksum computed according to an algorithm specified by
echo Microchip.  The second number is an overall sum of all program,
echo and data words plus only the active bits of the configuration words
echo (the Microchip algorithm ignores the data and includes all bits
echo of the configuration word).
echo.
echo.
PAUSE
CLS
echo.
echo The toolkit program SWAPHEX can convert one hex format to the
echo other.  There is no need to tell SWAPHEX what conversion to
echo do as it can differentiate between INHX16 and INHX8M automatically.
echo.
echo To illustrate SWAPHEX, here is the result of running
echo.
echo     SWAPHEX FIRST.HEX
echo.
SWAPHEX FIRST.HEX
echo.
echo.
PAUSE
CLS
echo The output of SWAPHEX can be stored in a file, SECOND.HEX for
echo example, by running
echo.
echo   SWAPHEX FIRST.HEX SECOND.HEX
echo.
SWAPHEX FIRST.HEX SECOND.HEX
echo Running
echo.
echo   WHATHEX SECOND.HEX
echo.
echo says:
echo.
WHATHEX SECOND.HEX
echo.
echo As expected, the file format is INHX16.  The other information
echo is exactly the same as it was for FIRST.HEX including both overall
echo checksums.
echo.
echo.
PAUSE
CLS
echo.
echo Running SWAPHEX on SECOND.HEX will convert it back to INHX8M:
echo.
SWAPHEX SECOND.HEX
echo.
echo You may recognise this as an INHX8M file, but it is not
echo identical to FIRST.HEX as you might expect.  That's because
echo the original file had a "hole" in the data area and SWAPHEX
echo has filled the hole with default values (the erased state of
echo a PIC).
echo.
echo.
PAUSE
CLS
echo.
echo Because files can look different but actually have the same
echo effect it is useful to have a program to compare two hex files
echo in a less literal and more intelligent way. (For various reasons
echo it's best not to rely on the overall checksums for this purpose.)
echo The toolkit program CMPHEX can compare two hex files even if they
echo have different formats.  FIRST.HEX and SECOND.HEX look different but
echo.
echo   CMPHEX FIRST.HEX SECOND.HEX
echo.
echo reports:
echo.
CMPHEX FIRST.HEX SECOND.HEX
echo.
PAUSE
CLS
echo.
echo Consumers of Microchip hex files (PIC programmers, disassemblers
echo and simulation programs for example) are not all created equal.
echo Some use INHX8M files, some INHX16 and some use proprietry formats;
echo some can use integrated program/data/ID/configuration files and
echo some can't.  This is where PICHEX can help.  As we have just
echo seen SWAPHEX can convert between styles.  Some other PICHEX
echo programs deal with conversions between integrated and separate
echo program and data files.
echo.
PAUSE
CLS
echo.
echo SPLITHEX can produce separate files from an integrated file.
echo As with any PICHEX program, running SPLITHEX without arguments
echo gives usage instructions:
echo.
echo     SPLITHEX
echo.
SPLITHEX
echo.
echo The [0] means 0 is an optional argument and can be left out.  If
echo 0 is specified then the addresses in datafile have 0 as origin
echo rather than the standard 2100.  SPLITHEX discards the ID and
echo configuration words.  To split the INHX16 file SECOND.HEX into a
echo program file (PROG.HEX say) and a standard data file (DATA.HEX say)
echo we can use:
echo.
echo.    SPLITHEX SECOND.HEX PROG.HEX DATA.HEX
echo.
SPLITHEX SECOND.HEX PROG.HEX DATA.HEX
echo.
PAUSE
CLS
echo.
echo WHATHEX can be used to look at the two files produced:
echo.
echo     WHATHEX PROG.HEX
echo.
WHATHEX PROG.HEX
echo.
echo     WHATHEX DATA.HEX
echo.
WHATHEX DATA.HEX
echo.
echo.
PAUSE
CLS
echo.
echo It is also possible to extract the data as a simple list of
echo hex bytes using HEX2LIST:
echo.
echo     HEX2LIST DATA.HEX
echo.
echo produces
echo.
HEX2LIST DATA.HEX
echo.
echo The 2100 at the head of the file specifies the data origin
echo (this should be in the range 2100-213F, or 0-3F).  In this form
echo it is easy to change the data using a text editor.
echo.
echo We can save this list in DATA.LIS by using:
echo.
echo     HEX2LIST DATA.HEX DATA.LIS
HEX2LIST DATA.HEX DATA.LIS
echo.
echo.
PAUSE
CLS
echo.
echo PICHEX provides an inverse of SPLITHEX that combines separate
echo program and data files; this is COMBHEX.
echo.
echo LIST2HEX performs the inverse function of HEX2LIST and can read
echo a free-format list of data to produce a hex file.
echo.
echo PICHEX can also add or replace ID records with ADDID and add
echo or replace a configuration word with ADDCFG.
echo.
echo To illustrate these programs we will use them to reconstruct a
echo copy of FIRST.HEX from the program-only hex file PROG.HEX and
echo the hex list DATA.LIS.
echo.
echo.
PAUSE
CLS
echo.
echo First we convert the hex list to a data file (NEWDATA.HEX):
echo.
echo     LIST2HEX DATA.LIS NEWDATA.HEX
echo.
LIST2HEX DATA.LIS NEWDATA.HEX
echo Then combine PROG.HEX and NEWDATA.HEX and save the result (THIRD.HEX).
echo.
echo     COMBHEX PROG.HEX NEWDATA.HEX THIRD.HEX
echo.
COMBHEX PROG.HEX NEWDATA.HEX THIRD.HEX
echo.
echo.
PAUSE
CLS
echo.
echo To check progress we can compare the intermediate file THIRD.HEX
echo with FIRST.HEX:
echo.
echo     CMPHEX THIRD.HEX FIRST.HEX
echo.
CMPHEX THIRD.HEX FIRST.HEX
echo.
echo The ID and config are different so we must add the appropriate ID
echo and config records to THIRD.HEX.
echo.
echo.
PAUSE
CLS
echo.
echo The ID bytes and config word of FIRST.HEX can be determined using
echo WHATHEX again.
echo.
echo     WHATHEX FIRST.HEX
echo.
WHATHEX FIRST.HEX
echo.
echo Now we can add the required ID bytes (A, B, C, D) to THIRD.HEX and
echo put the result in FOURTH.HEX:
echo.
echo     ADDID A B C D THIRD.HEX FOURTH.HEX
echo.
ADDID A B C D THIRD.HEX FOURTH.HEX
echo.
echo.
PAUSE
CLS
echo.
echo WHATHEX said that the desired config was PWRT_ON (power-up timer
echo enabled), WDT_ON (watchdog enabled) and XT_OSC (crystal oscillator).
echo The correct config can be added to FOURTH.HEX to produce the final
echo file FIFTH.HEX using:
echo.
echo     ADDCFG PWX FOURTH.HEX FIFTH.HEX
echo.
ADDCFG PWX FOURTH.HEX FIFTH.HEX
echo.
echo.
PAUSE
CLS
echo.
echo Now FIFTH.HEX should be identical to FIRST.HEX and
echo.
echo     CMPHEX FIFTH.HEX FIRST.HEX
echo.
echo confirms
echo.
CMPHEX FIFTH.HEX FIRST.HEX
echo.
echo.
PAUSE
CLS
echo.
echo In fact FIFTH.HEX and FIRST.HEX are not completely identical as can
echo be seen by comparing the data size and the Microchip checksums of both:
echo.
echo     WHATHEX FIFTH.HEX
echo.
WHATHEX FIFTH.HEX
echo.
echo     WHATHEX FIRST.HEX
echo.
WHATHEX FIRST.HEX
echo.
echo.
PAUSE
CLS
echo.
echo The differences are not important as CMPHEX has verified that the
echo the files are effectively the same.  The checksum difference is
echo due to the fact that the Microchip algorithm includes all bits of
echo the configuration word in the checksum.  The alternative checksum
echo only uses the active bits of the config word and this is the same
echo for both files.
echo.
echo.
PAUSE
CLS
echo.
echo The PICHEX toolkit can convert hex files to "John Morrison"
echo binary format (HEX2JM) and back again (JM2HEX).  We'll try that
echo with FIRST.HEX first creating FIRST.BIN then converting FIRST.BIN
echo to SIXTH.HEX.
echo.
echo     HEX2JM FIRST.HEX FIRST.BIN
HEX2JM FIRST.HEX FIRST.BIN
echo.
echo     JM2HEX FIRST.BIN SIXTH.HEX
JM2HEX FIRST.BIN SIXTH.HEX
echo.
echo Comparing the result with the original file with
echo.
echo     CMPHEX SIXTH.HEX FIRST.HEX
echo.
echo says:
echo.
CMPHEX SIXTH.HEX FIRST.HEX
echo.
echo The ID bytes are not stored in the John Morrison file and therefore
echo could not recovered.
echo.
PAUSE
CLS
echo.
echo Cleaning up temporary files.
DEL SECOND.HEX
DEL PROG.HEX
DEL DATA.HEX
DEL DATA.LIS
DEL NEWDATA.HEX
DEL THIRD.HEX
DEL FOURTH.HEX
DEL FIFTH.HEX
DEL FIRST.BIN
DEL SIXTH.HEX
CLS
echo.
echo.
echo.
echo                     That's all folks ...
echo.
echo.
echo.
