#
# Turbo-C makefile to build the PICHEX package in toolkit form.
# Use "make -f toolkit dist" to rebuild the distribution with
# pkzip.
#

VER = 01
CC = tcc
ZIP = pkzip

CFILES1  = swaphex.c combhex.c jm2hex.c hex2jm.c
CFILES2  = splithex.c whathex.c cmphex.c addcfg.c addid.c
CFILES3  = list2hex.c hex2list.c hex.c misc.c

CFILES   = $(CFILES1) $(CFILES2) $(CFILES3)

HFILES   = hex.h misc.h pichex.h

EXEFILES1 = swaphex.exe combhex.exe jm2hex.exe hex2jm.exe
EXEFILES2 = splithex.exe whathex.exe cmphex.exe addcfg.exe
EXEFILES3 = addid.exe list2hex.exe hex2list.exe

EXEFILES = $(EXEFILES1) $(EXEFILES2) $(EXEFILES3)

DOCS     = readme.1st pichex.txt files.txt first.hex demo.bat

ZIPALL   = pichex$(VER).zip

ZIPEXTRA = src.zip vpp.zip

MAKEFILES = toolkit.mak lib.mak

SRCFILES = $(CFILES) $(HFILES) $(MAKEFILES) src.txt

VPPFILES = vpp.c vpp.txt vpp.mak vpp.exe

OBJS     = hex.obj misc.obj


all:            $(EXEFILES)

dist:           $(ZIPALL)
		

swaphex.exe:    swaphex.obj $(OBJS)
		$(CC) swaphex.obj hex.obj misc.obj

combhex.exe:    combhex.obj $(OBJS)
		$(CC) combhex.obj $(OBJS)

jm2hex.exe:     jm2hex.obj $(OBJS)
		$(CC) jm2hex.obj $(OBJS)

hex2jm.exe:     hex2jm.obj $(OBJS)
		$(CC) hex2jm.obj $(OBJS)

splithex.exe:   splithex.obj $(OBJS)
		$(CC) splithex.obj $(OBJS)

whathex.exe:    whathex.obj $(OBJS)
		$(CC) whathex.obj $(OBJS)

cmphex.exe:     cmphex.obj $(OBJS)
		$(CC) cmphex.obj $(OBJS)

addcfg.exe:     addcfg.obj $(OBJS)
		$(CC) addcfg.obj $(OBJS)

addid.exe:      addid.obj $(OBJS)
		$(CC) addid.obj $(OBJS)

list2hex.exe:   list2hex.obj $(OBJS)
		$(CC) list2hex.obj $(OBJS)

hex2list.exe:   hex2list.obj $(OBJS)
		$(CC) hex2list.obj $(OBJS)

vpp.exe:	vpp.obj $(OBJS)
		$(CC) vpp.obj $(OBJS)

pichex$(VER).zip: $(EXEFILES) $(ZIPEXTRA) $(DOCS)
		  $(ZIP) $(ZIPALL) $(EXEFILES1)
		  $(ZIP) $(ZIPALL) $(EXEFILES2)
		  $(ZIP) $(ZIPALL) $(EXEFILES3)
		  $(ZIP) $(ZIPALL) $(DOCS)
		  $(ZIP) $(ZIPALL) $(ZIPEXTRA)

src.zip:        $(SRCFILES)
		$(ZIP) src.zip $(CFILES1)
		$(ZIP) src.zip $(CFILES2)           
		$(ZIP) src.zip $(CFILES3)
		$(ZIP) src.zip $(HFILES)
		$(ZIP) src.zip $(MAKEFILES)
		$(ZIP) src.zip src.txt
		
vpp.zip:        $(VPPFILES)
		$(ZIP) vpp.zip $(VPPFILES)

