/*
 * addcfg.c
 *
 * Adds/replaces a config record in a Microchip hex file.
 *
 *
 * Usage: addcfg cftext infile [outfile]
 *
 * If outfile is not specified stdout is used.
 * cfgtext is a string containing one or more of:
 *
 *     'c' (code protect)
 *     'p' (power-up timer enable)
 *     'w' (watchdog enable)
 *     'l' (LP oscillator)
 *     'x' (XT "")
 *     'h' (HS "")
 *     'r' (RC "")
 *
 *
 * Revision history:
 *
 * 15-Jul-1996: V-0.0; created
 * 18-Jul-1996: V-0.1; changed to support LIBVERSION
 *
 * Copyright (C) 1996 David Tait.  All rights reserved.
 * Permission is granted to use, modify, or redistribute this software
 * so long as it is not sold or exploited for profit.
 *
 * THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include "hex.h"
#include "misc.h"

static char *pname = "addcfg";
#ifndef LIBVERSION
char *usage = "Usage: addcfg cftext infile [outfile]\n\
\tcftext from {cpwlxhr}";
#endif
static char *version = "addcfg.c V-0.1  Copyright (C) 1996 David Tait";


static int setcfg(char *cp)
{
    config = 0x10;

    while ( *cp )
	switch ( *cp++ ) {
	    case 'c': case 'C':
		config &= ~CP;
		break;
	    case 'p': case 'P':
		config |= PWRT;
		break;
	    case 'w': case 'W':
		config |= WDT;
		break;
	    case 'l': case 'L':
		config |= LP_OSC;
		break;
	    case 'x': case 'X':
		config |= XT_OSC;
		break;
	    case 'h': case 'H':
		config |= HS_OSC;
		break;
	    case 'r': case 'R':
		config |= RC_OSC;
		break;
	    default:
		return -1;
	}
	return 0;
}



int addcfg(FILE *ifp, FILE *ofp, char *cftxt)
{
    int e;

    erasehex(PSIZE, DSIZE, WSIZE);
    format = UNKNOWN;
    if ( (e = loadhex(ifp, PSIZE, DSIZE, IBASE, CBASE, DBASE)) != 0 )
	telluser(pname,"While loading infile:",e);
    if ( setcfg(cftxt) < 0 )
	quit(pname,"Illegal config option");;
    dumphex(ofp, pmlast+1, dmlast-DBASE+1, id, CBASE, DBASE);
    return 0;
}


#ifndef LIBVERSION

int main(int argc, char *argv[])
{
    FILE *ifp, *ofp = stdout;

    if ( argc <= 2 )
	quit(pname,usage);

    if ( (ifp = fopen(argv[2],"r")) == NULL )
	quit(pname,"Can't open infile");

    if ( argc > 3 )
	if ( (ofp = fopen(argv[3],"w")) == NULL )
	    quit(pname,"Can't create outfile");

    addcfg(ifp, ofp, argv[1]);

    return EXIT_SUCCESS;
}

#endif /* LIBVERSION */