/*
 * addid.c
 *
 * Adds/replaces an ID record in a Microchip hex file.
 *
 *
 * Usage: addid id1 id2 id3 id4 infile [outfile]
 *
 * If outfile is not specified stdout is used.  The id words id1-id4
 * are specified as hex strings.
 *
 *
 * Revision history:
 *
 * 15-Jul-1996: V-0.0; created
 * 18-Jul-1996: V-0.1; changed to support LIBVERSION
 *
 * Copyright (C) 1996 David Tait.  All rights reserved.
 * Permission is granted to use, modify, or redistribute this software
 * so long as it is not sold or exploited for profit.
 *
 * THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include "hex.h"
#include "misc.h"

static char *pname = "addid";
#ifndef LIBVERSION
char *usage = "Usage: addid id1 id2 id3 id4 infile [outfile]";
#endif
static char *version = "addid.c V-0.1  Copyright (C) 1996 David Tait";


static int setid(char *hexid[])
{
    int i, w;

    for ( i=0; i<4; ++i ) {
	if ( (w = hex2int(4,hexid[i])) < 0 )
	    return -1;
	idbuf[i] = w;
    }
    return 0;
}


int addid(FILE *ifp, FILE *ofp, char *hexid[])
{
    int e;

    erasehex(PSIZE, DSIZE, WSIZE);
    format = UNKNOWN;
    if ( (e = loadhex(ifp, PSIZE, DSIZE, IBASE, CBASE, DBASE)) != 0 )
	telluser(pname,"While loading infile:",e);
    if ( setid(hexid) < 0 )
	quit(pname,"ID word is not hex");;
    dumphex(ofp, pmlast+1, dmlast-DBASE+1, IBASE, cf, DBASE);
    return 0;
}


#ifndef LIBVERSION

int main(int argc, char *argv[])
{
    FILE *ifp, *ofp = stdout;

    if ( argc <= 5 )
	quit(pname,usage);

    if ( (ifp = fopen(argv[5],"r")) == NULL )
	quit(pname,"Can't open infile");

    if ( argc > 6 )
	if ( (ofp = fopen(argv[6],"w")) == NULL )
	    quit(pname,"Can't create outfile");

    addid(ifp, ofp, &argv[1]);

    return EXIT_SUCCESS;
}

#endif  /* LIBVERSION */