/*
 * cmphex.c
 *
 * Compares two Microchip hex files.  The files are not compared directly
 * (they can even be different formats), instead their programming effect
 * is compared.
 *
 *
 * Usage: cmphex hexfile1 hexfile2
 *
 *
 * Revision history:
 *
 * 15-Jul-1996: V-0.0; created
 * 18-Jul-1996: V-0.1; changed to use support LIBVERSION
 *
 * Copyright (C) 1996 David Tait.  All rights reserved.
 * Permission is granted to use, modify, or redistribute this software
 * so long as it is not sold or exploited for profit.
 *
 * THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include "hex.h"
#include "misc.h"

static char *pname = "cmphex";
#ifndef LIBVERSION
char *usage = "Usage: cmphex hexfile1 hexfile2";
#endif
static char *version = "cmphex.c V-0.1  Copyright (C) 1996 David Tait";


static U16 a_progbuf[PSIZE];
static U16 a_databuf[DSIZE];
static U16 a_idbuf[4];
static U16 a_config;


static void copypic(void)
{
    int i;

    for ( i=0; i<PSIZE; ++i )
	a_progbuf[i] = progbuf[i];

    for ( i=0; i<DSIZE; ++i )
	a_databuf[i] = databuf[i];

    for ( i=0; i<4; ++i )
	a_idbuf[i] = idbuf[i];

    a_config = config;
}


static int differ(U16 *ap, U16 *bp, int nw)
{
    int i, nd = 0;

    for ( i=0; i<nw; ++i )
	if ( *ap++ != *bp++ )
	    ++nd;
    return nd;
}


int cmphex(FILE *afp, FILE *bfp, int diffs[])
{
    int e, tmpfmt;

    erasehex(PSIZE, DSIZE, WSIZE);
    format = UNKNOWN;
    if ( (e = loadhex(afp, PSIZE, DSIZE, IBASE, CBASE, DBASE)) != 0 )
	telluser(pname,"While loading first file:",e);
    tmpfmt = format;

    copypic();

    erasehex(PSIZE, DSIZE, WSIZE);
    format = UNKNOWN;
    if ( (e = loadhex(bfp, PSIZE, DSIZE, IBASE, CBASE, DBASE)) != 0 )
	telluser(pname,"While loading second file:",e);

    diffs[0] = differ(a_progbuf, progbuf, PSIZE);
    diffs[1] = differ(a_databuf, databuf, DSIZE);
    diffs[2] = differ(a_idbuf, idbuf, 4);
    diffs[3] = ((a_config&CMASK) != (config&CMASK));
    diffs[4] = (tmpfmt != format);
    return 0;
}


#ifndef LIBVERSION

int main(int argc, char *argv[])
{
    FILE *afp, *bfp = stdout;
    int diffs[5], df;

    if ( argc <= 2 )
	quit(pname,usage);

    if ( (afp = fopen(argv[1],"r")) == NULL )
	quit(pname,"Can't open hexfile1");

    if ( (bfp = fopen(argv[2],"r")) == NULL )
	quit(pname,"Can't open hexfile2");

    cmphex(afp, bfp, diffs);

    if ( diffs[4] )
	printf("Comparing files with different formats:\n");
    else
	printf("Comparing files with the same format:\n");

    df = 0;
    if ( diffs[0] > 0 ) {
	printf(" Program differs in %d locations\n",diffs[0]);
	++df;
    }
    if ( diffs[1] > 0 ) {
	printf(" Data differs in %d locations\n",diffs[1]);
	++df;
    }
    if ( diffs[2] ) {
	printf(" Different ID words\n");
	++df;
    }
    if ( diffs[3] ) {
	printf(" Different config\n");
	++df;
    }

    if ( !df )
	printf(" No differences\n");

    return EXIT_SUCCESS;
}

#endif  /* LIBVERSION */