/*
 * combhex.c
 *
 * Produces a single Microchip hex file from two others: one file
 * provides the program, ID and config words, and the other provides
 * the EEDATA bytes.  The input files can be in different formats
 * (either INHX8M or INHX16) but INHX8M is used for the output file.
 *
 *
 * Usage: combhex progfile datafile [outfile]
 *
 * If outfile is not specified stdout is used.
 *
 *
 * Revision history:
 *
 * 14-Jul-1996: V-0.0; created
 * 18-Jul-1996: V-0.1; changed to support LIBVERSION
 *
 * Copyright (C) 1996 David Tait.  All rights reserved.
 * Permission is granted to use, modify, or redistribute this software
 * so long as it is not sold or exploited for profit.
 *
 * THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "hex.h"
#include "misc.h"

static char *pname = "combhex";
#ifndef LIBVERSION
char *usage = "Usage: combhex progfile datafile [outfile]";
#endif
static char *version = "combhex.c V-0.1  Copyright (C) 1996 David Tait";


int combhex(FILE *pfp, FILE *dfp, FILE *ofp)
{
    int i, e, pmtmp, idtmp, cftmp;

    erasehex(PSIZE, DSIZE, WSIZE);      /* "erase" buffers */
    format = UNKNOWN;                   /* auto-identify input style */
    if ( (e = loadhex(pfp, PSIZE, 0, IBASE, CBASE, DBASE)) != 0 )
	telluser(pname,"While loading progfile:",e);

    pmtmp = pmlast;
    idtmp = id;
    cftmp = cf;

    format = UNKNOWN;			   /* first try loading at DBASE */
    if ( (e = loadhex(dfp, 0, DSIZE, IBASE, CBASE, DBASE)) != 0 )
	if ( e < 0 )
	    telluser(pname,"While loading datafile:",e);

    if ( dmlast < 0 ) {                 /* no data found, try loading at 0 */
	fseek(dfp, 0L, 0);
	if ( (e = loadhex(dfp, 0, DSIZE, IBASE, CBASE, 0)) != 0 )
	    if ( e < 0 )
		telluser(pname,"While loading datafile:",e);
	dmlast += DBASE;
    }

    if ( e != 0 )
	telluser(pname,"While loading datafile:", e);

    if ( id ) {
	telluser(pname,"(Warning) ID set by data file",0);
	idtmp = id;
    }

    if ( cf ) {
	telluser(pname,"(Warning) config set by data file",0);
	cftmp = cf;
    }

    format = INHX8M;
    dumphex(ofp, pmtmp+1, dmlast-DBASE+1, idtmp, cftmp, DBASE);
    return 0;
}


#ifndef LIBVERSION

int main(int argc, char *argv[])
{
    FILE *pfp, *dfp, *ofp = stdout;

    if ( argc <= 2 )
	quit(pname,usage);

    if ( (pfp = fopen(argv[1],"r")) == NULL )
	quit(pname,"Can't open progfile");

    if ( (dfp = fopen(argv[2],"r")) == NULL )
	quit(pname,"Can't open datafile");

    if ( argc > 3 )
	if ( (ofp = fopen(argv[3],"w")) == NULL )
	    quit(pname,"Can't create outfile");

    combhex(pfp, dfp, ofp);

    return EXIT_SUCCESS;
}

#endif  /* LIBVERSION */