/*
 * hex2jm.c
 *
 * Converts Microchip INHX8M or INHX16 files to John Morrison binary
 * format.  (Only PIC16C84 files are supported.)
 *
 *
 * Usage: hex2jm hexfile binfile
 *
 *
 * Revision history:
 *
 * 14-Jul-1996: V-0.0; created by guessing JM format
 * 18-Jul-1996: V-0.1; changed to support LIBVERSION
 *
 * Copyright (C) 1996 David Tait.  All rights reserved.
 * Permission is granted to use, modify, or redistribute this software
 * so long as it is not sold or exploited for profit.
 *
 * THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include "hex.h"
#include "misc.h"

static char *pname = "hex2jm";
#ifndef LIBVERSION
char *usage = "Usage: hex2jm hexfile binfile";
#endif
static char *version = "hex2jm.c V-0.1  Copyright (C) 1996 David Tait";
char buf[16];


static void writebin(FILE *fp)
{
    int i, c;
    U16 *bp;

    bp = progbuf;
    for ( i=0; i < PSIZE; ++i) {
	*bp = SWAB(*bp);
	fwrite(bp++, 2, 1, fp);
    }

    buf[0] = config;
    buf[1] = 0xE9;			/* magic number? */
    fwrite(buf, 16, 1, fp);

    bp = databuf;
    for ( i=0; i < DSIZE; ++i)
	putc(*bp++,fp);

    buf[0] = buf[1] = 0;
    fwrite(buf, 16, 1, fp);
}


int hex2jm(FILE *hfp, FILE *bfp)
{
    int e;

    erasehex(PSIZE, DSIZE, WSIZE);
    format = UNKNOWN;
    if ( (e = loadhex(hfp, PSIZE, DSIZE, IBASE, CBASE, DBASE)) != 0 )
	telluser(pname,"While loading hexfile:",e);
    writebin(bfp);
    return 0;
}


#ifndef LIBVERSION

int main(int argc, char *argv[])
{
    FILE *ifp, *ofp = stdout;

    if ( argc < 3 )
	quit(pname,usage);

    if ( (ifp = fopen(argv[1],"r")) == NULL )
	quit(pname,"Can't open hexfile file");

    if ( (ofp = fopen(argv[2],"wb")) == NULL )
	quit(pname,"Can't create binfile");

    hex2jm(ifp, ofp);

    return EXIT_SUCCESS;
}

#endif  /* LIBVERSION */