/*
 * hex2list.c
 *
 * Dumps the EEDATA contents of Microchip hex file as a list of hex
 * bytes.  The first line of the list is the EEDATA origin.  If the
 * file contains no data then the first 64 bytes of program
 * memory are dumped.
 *
 *
 *
 * Usage: hex2list hexfile [listfile]
 *
 * If listfile is not specified stdout is used.
 *
 *
 * Revision history:
 *
 * 15-Jul-1996: V-0.0; created
 * 18-Jul-1996: V-0.1; changed to support LIBVERSION
 *
 * Copyright (C) 1996 David Tait.  All rights reserved.
 * Permission is granted to use, modify, or redistribute this software
 * so long as it is not sold or exploited for profit.
 *
 * THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include "hex.h"
#include "misc.h"

static char *pname = "hex2list";
#ifndef LIBVERSION
char *usage = "Usage: hex2list hexfile [listfile]";
#endif
static char *version = "hex2list.c V-0.1  Copyright (C) 1996 David Tait";


static void listbuf(U16 *bp, int origin, FILE *fp)
{
    int i;

    fprintf(fp,"%04X",origin);
    for ( i=0; i<DSIZE; ++i )
	fprintf(fp,"%c%02X",(i%16)? ' ': '\n',(*bp++)&0xFF);
    fprintf(fp,"\n");
}


int hex2list(FILE *hfp, FILE *lfp)
{
    int e, data;
    U16 *bp;

    erasehex(PSIZE, DSIZE, WSIZE);

    format = UNKNOWN;

    if ( (e = loadhex(hfp, PSIZE, DSIZE, IBASE, CBASE, DBASE)) != 0 )
	telluser(pname,"While loading hexfile:",e);

    data = (dmlast >= 0 );
    listbuf(data? databuf: progbuf, data? DBASE: 0, lfp);
    return 0;
}


#ifndef LIBVERSION

int main(int argc, char *argv[])
{
    FILE *hfp, *lfp = stdout;

    if ( argc <= 1 )
	quit(pname,usage);

    if ( (hfp = fopen(argv[1],"r")) == NULL )
	quit(pname,"Can't open hexfile");

    if ( argc > 2 )
	if ( (lfp = fopen(argv[2],"w")) == NULL )
	    quit(pname,"Can't create listfile");

    hex2list(hfp, lfp);

    return EXIT_SUCCESS;
}

#endif  /* LIBVERSION */
