/*
 * jm2hex.c
 *
 * Converts John Morrison binary files to Microchip INHX8M format.
 * (Only PIC16C84 files are supported.)
 *
 *
 * Usage: jm2hex binfile [hexfile]
 *
 * If hexfile is not specified stdout is used.
 *
 *
 * Revision history:
 *
 * 14-Jul-1996: V-0.0; created by guessing JM format
 * 16-Jul-1996: V-0.1; changed to support LIBVERSION
 *
 * Copyright (C) 1996 David Tait.  All rights reserved.
 * Permission is granted to use, modify, or redistribute this software
 * so long as it is not sold or exploited for profit.
 *
 * THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include "hex.h"
#include "misc.h"

static char *pname = "jm2hex";
#ifndef LIBVERSION
char *usage = "Usage: jm2hex binfile [hexfile]";
#endif
static char *version = "jm2hex.c V-0.1  Copyright (C) 1996 David Tait";

static char buf[16];

static int readbin(FILE *fp)
{
    int i, c;
    U16 *bp;

    bp = progbuf;
    for ( i=0; i < PSIZE; ++i) {
	if ( fread(bp, 2, 1, fp) != 1 )
	    return EOF;
	*bp++ = SWAB(*bp);
    }

    if ( fread(buf, 16, 1, fp) != 1 )
	return EOF;
    config = buf[0]&0xFF;

    bp = databuf;
    for ( i=0; i < DSIZE; ++i) {
	if ( (c = getc(fp)) == EOF )
	    return EOF;
	*bp++ = c;
    }

    if ( fread(buf, 16, 1, fp) != 1 )
	return EOF;

    return 0;
}


int jm2hex(FILE *bfp, FILE *hfp)
{

    if ( readbin(bfp) == EOF )
	quit(pname,"Short binary file");
    format = INHX8M;
    dumphex(hfp, PSIZE, DSIZE, 0, CBASE, DBASE);  /* No ID words? */
    return 0;
}


#ifndef LIBVERSION

int main(int argc, char *argv[])
{
    FILE *ifp, *ofp = stdout;

    if ( argc <= 1 )
	quit(pname,usage);

    if ( (ifp = fopen(argv[1],"rb")) == NULL )
	quit(pname,"Can't open binary file");

    if ( argc > 2 )
	if ( (ofp = fopen(argv[2],"w")) == NULL )
	    quit(pname,"Can't create hexfile");

    jm2hex(ifp, ofp);

    return EXIT_SUCCESS;
}

#endif  /* LIBVERSION */