/*
 * list2hex.c
 *
 * Converts a list of hex bytes to an EEDATA file in INHX8M format.  The
 * first entry in the list is the origin used for the hexfile and can
 * be either between 0 and 0x3F or between 0x2100 and 0x213F.  Extra
 * bytes in the list are silently ignored.
 *
 *
 * Usage: list2hex listfile [hexfile]
 *
 * If hexfile is not specified stdout is used.
 *
 *
 * Revision history:
 *
 * 15-Jul-1996: V-0.0; created
 * 18-Jul-1996: V-0.1; changed to support LIBVERSION
 *
 * Copyright (C) 1996 David Tait.  All rights reserved.
 * Permission is granted to use, modify, or redistribute this software
 * so long as it is not sold or exploited for profit.
 *
 * THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include "hex.h"
#include "misc.h"

static char *pname = "list2hex";
#ifndef LIBVERSION
char *usage = "Usage: list2hex listfile [hexfile]";
#endif
static char *version = "list2hex.c V-0.1  Copyright (C) 1996 David Tait";

#define FSIZE  (5*DSIZE)   	/* probably large enough */

static char buf[FSIZE+1];


static int readlist(FILE *fp)
{
    char *bp = buf;
    int c, n = 0;

    while ( (c = getc(fp)) != EOF && n < FSIZE ) {
	*bp++ = c;
	++n;
    }
    if ( n == FSIZE )
	return -1;

    *bp = '\0';

    return n;
}


static int readhex(int n)
{
    static char *bp = buf;
    int c, w;

    if ( *bp == '\0' )
	return 0;

    while ( (c = *bp) == ' ' || c == '\n' )	 /* skip white space */
	bp++;

    w = hex2int(n,bp);

    while ( (c = *bp) != ' ' && c != '\n' && c != '\0' )  /* skip hex */
	bp++;

    return w;
}


int list2hex(FILE *lfp, FILE *hfp)
{
    int origin, i, b, e;

    erasehex(PSIZE, DSIZE, WSIZE);

    format = INHX8M;

    if ( readlist(lfp) < 0 )
	quit(pname,"List file too large, re-format");

    origin = readhex(4);
    if ( (origin >= DSIZE && origin < DBASE)
	 || origin < 0 || origin > (DBASE+DSIZE) )
	quit(pname,"Bad origin in list file");


    i = (origin >= DBASE)? origin-DBASE: 0;
    for ( ; i < DSIZE; ++i ) {
	if ( (b = readhex(2)) < 0 )
	    quit(pname,"Bad entry in list");
	databuf[i] = b;
    }

    dumphex(hfp, 0, DSIZE, 0, 0, (origin>=DBASE)? DBASE: 0);
    return 0;
}


#ifndef LIBVERSION

int main(int argc, char *argv[])
{
    FILE *ifp, *ofp = stdout;

    if ( argc <= 1 )
	quit(pname,usage);

    if ( (ifp = fopen(argv[1],"r")) == NULL )
	quit(pname,"Can't open listfile");

    if ( argc > 2 )
	if ( (ofp = fopen(argv[2],"w")) == NULL )
	    quit(pname,"Can't create hexfile");

    list2hex(ifp, ofp);

    return EXIT_SUCCESS;
}

#endif  /* LIBVERSION */