/*
 * splithex.c
 *
 * Splits a Microchip hex file into two others: one file containing
 * only the program words and the other containing only EEDATA bytes.
 * The output files are written in the same format (either INHX8M or
 * INHX16) as the input.  The ID words and config word are discarded.
 *
 *
 * Usage: splithex hexfile progfile datafile [0]
 *
 * If last parameter is 0 then an origin of 0 rather than 0x2100 is
 * used for the EEDATA hex file.
 *
 *
 * Revision history:
 *
 * 15-Jul-1996: V-0.0; created
 * 18-Jul-1996: V-0.1; changed to support LIBVERSION
 *
 * Copyright (C) 1996 David Tait.  All rights reserved.
 * Permission is granted to use, modify, or redistribute this software
 * so long as it is not sold or exploited for profit.
 *
 * THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "hex.h"
#include "misc.h"

static char *pname = "splithex";
#ifndef LIBVERSION
char *usage = "Usage: splithex hexfile progfile datafile [0]";
#endif
static char *version = "splithex.c V-0.1  Copyright (C) 1996 David Tait";


int splithex(FILE *hfp, char *pfn, char *dfn, int origin)
{
    int e;
    FILE *pfp, *dfp;

    erasehex(PSIZE, DSIZE, WSIZE);	/* "erase" buffers */

    format = UNKNOWN;			/* auto-identify input style */

    if ( (e = loadhex(hfp, PSIZE, DSIZE, IBASE, CBASE, DBASE)) != 0 )
	telluser(pname,"While loading hexfile:",e);

    if ( pmlast >= 0 ) {
	if ( (pfp = fopen(pfn,"w")) == NULL )
	    quit(pname,"Can't create program file");
	dumphex(pfp, pmlast+1, 0, 0, 0, 0);
    } else
	telluser(pname,"No program words!  Program file not created",0);

    if ( dmlast >= 0 ) {
	if ( (dfp = fopen(dfn,"w")) == NULL )
	    quit(pname,"Can't create data file");
	dumphex(dfp, 0, dmlast-DBASE+1, 0, 0, origin);
    } else
	telluser(pname,"No data words!  Data file not created",0);

    return 0;
}


#ifndef LIBVERSION

int main(int argc, char *argv[])
{
    FILE *hfp;
    int origin;

    if ( argc <= 3 )
	quit(pname,usage);

    if ( (hfp = fopen(argv[1],"r")) == NULL )
	quit(pname,"Can't open hexfile");

    origin = (argc > 4 && argv[4][0] == '0')? 0: DBASE;

    splithex(hfp, argv[2], argv[3], origin);

    return EXIT_SUCCESS;
}

#endif  /* LIBVERSION */