/*
 * swaphex.c
 *
 * Converts Microchip hex files in INHX8M format to INHX16 format
 * and vice-versa.
 *
 *
 * Usage: swaphex infile [outfile]
 *
 * If outfile is not specified stdout is used.
 *
 *
 * Revision history:
 *
 * 11-Jul-1996: V-0.0; created
 * 18-Jul-1996: V-0.1; changed to support LIBVERSION
 *
 * Copyright (C) 1996 David Tait.  All rights reserved.
 * Permission is granted to use, modify, or redistribute this software
 * so long as it is not sold or exploited for profit.
 *
 * THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include "hex.h"
#include "misc.h"

static char *pname = "swaphex";
#ifndef LIBVERSION
char *usage = "Usage: swaphex infile [outfile]";
#endif
static char *version = "swaphex.c V-0.1  Copyright (C) 1996 David Tait";


int swaphex(FILE *ifp, FILE *ofp)
{
    int i, e;

    erasehex(PSIZE, DSIZE, WSIZE);   	/* "erase" buffers */

    format = UNKNOWN;			/* auto-identify input style */

    if ( (e = loadhex(ifp, PSIZE, DSIZE, IBASE, CBASE, DBASE)) != 0 )
	telluser(pname,"While loading infile:",e);

    format = (format == INHX8M)? INHX16: INHX8M;	/* swap styles */

    dumphex(ofp, pmlast+1, dmlast-DBASE+1, id, cf, DBASE);
    return 0;
}


#ifndef LIBVERSION

int main(int argc, char *argv[])
{
    FILE *ifp, *ofp = stdout;

    if ( argc <= 1 )
	quit(pname,usage);

    if ( (ifp = fopen(argv[1],"r")) == NULL )
	quit(pname,"Can't open infile");

    if ( argc > 2 )
	if ( (ofp = fopen(argv[2],"w")) == NULL )
	    quit(pname,"Can't create outfile");

    swaphex(ifp, ofp);

    return EXIT_SUCCESS;
}

#endif  /* LIBVERSION */