/*
 * vpp.c
 *
 * Virtual pp.  A frontend to pp V-0.3 that allows it to use integrated
 * hex files in either INHX8M or INHX16 format.  ID words are ignored.
 * If config details are in the file they are used otherwise pp will
 * prompt for them.
 *
 * Just for fun as it's just as easy to add the routines in hex.c to
 * the original pp.c.
 *
 * Usage: vpp hexfile
 *
 *
 * Revision history:
 *
 * 15-Jul-1996: V-0.0; created
 *
 * Copyright (C) 1996 David Tait.  All rights reserved.
 * Permission is granted to use, modify, or redistribute this software
 * so long as it is not sold or exploited for profit.
 *
 * THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include "hex.h"
#include "misc.h"


char *pname = "vpp";
char *usage = "Usage: vpp hexfile";
static char *version = "vpp.c V-0.0  Copyright (C) 1996 David Tait";

char *oscopts[] = {"l","x","h","r"};
char ppcmd[80];


void addtocmd(char *s)
{
    static char *cp = ppcmd;

    while ( *s )
	*cp++ = *s++;
    *cp = '\0';
}


void ppopts(void)
{
    addtocmd(" -");
    addtocmd(oscopts[config&OSC]);
    if ( (config&CP) == 0)
	addtocmd("c");
    if( config&PWRT )
	addtocmd("p");
    if ( config&WDT )
	addtocmd("w");
    if ( format == INHX8M )
	addtocmd("8");
}


void vpp(FILE *hfp)
{
    int e, ppexists = 0;
    FILE *pfp, *dfp, *tmpfp;

    erasehex(PSIZE, DSIZE, WSIZE);

    format = UNKNOWN;
    if ( (e = loadhex(hfp, PSIZE, DSIZE, IBASE, CBASE, DBASE)) != 0 )
	telluser(pname,"While loading hexfile:",e);

    if ( pmlast >= 0 ) {
	if ( (pfp = fopen("________.PRG","w")) == NULL )
	    quit(pname,"Can't create temporary program file");
	dumphex(pfp, pmlast+1, 0, 0, 0, 0);
	fclose(pfp);
    } else
	quit(pname,"No program words!");

    if ( dmlast >= 0 ) {
	if ( (dfp = fopen("________.DAT","w")) == NULL )
	    quit(pname,"Can't create temporary data file");
	dumphex(dfp, 0, dmlast-DBASE+1, 0, 0, 0);    /* dump data at 0 */
	fclose(dfp);
    }

    if ( (tmpfp = fopen("pp.exe","r")) != NULL ) {  /* look for pp.exe */
	++ppexists;
	fclose(tmpfp);
    }

    addtocmd("pp");
    if ( cf )
	ppopts();
    addtocmd(" ________.PRG");
    if ( dmlast >= 0 )
	addtocmd(" ________.DAT");
    if ( ppexists)
	system(ppcmd);
    else
	printf("\nWant to run:\n\n\t%s\n\nbut pp.exe is missing.\n",ppcmd);
    unlink("________.PRG");
    unlink("________.DAT");
}


int main(int argc, char *argv[])
{
    FILE *hfp;

    if ( argc <= 1 )
	quit(pname,usage);

    if ( (hfp = fopen(argv[1],"r")) == NULL )
	quit(pname,"Can't open hexfile");

    vpp(hfp);

    return EXIT_SUCCESS;
}
