/*
 * whathex.c
 *
 * Describes the contents of a Microchip hex file.  The length of the
 * program and data areas is given, the ID words are printed and the
 * configuration is analysed.  An overall checksum is calculated two
 * ways: the Microchip way, and one that includes the data area and
 * only the active bits of the configuration word.
 *
 *
 *
 * Usage: whathex hexfile
 *
 *
 * Revision history:
 *
 * 15-Jul-1996: V-0.0; created
 * 18-Jul-1996: V-0.1; changed to support LIBVERSION
 *
 * Copyright (C) 1996 David Tait.  All rights reserved.
 * Permission is granted to use, modify, or redistribute this software
 * so long as it is not sold or exploited for profit.
 *
 * THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND,
 * EITHER EXPRESSED OR IMPLIED.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "hex.h"
#include "misc.h"

static char *pname = "whathex";
#ifndef LIBVERSION
char *usage = "Usage: whathex hexfile";
#endif
static char *version = "whathex.c V-0.1  Copyright (C) 1996 David Tait";

int whathex(FILE *hfp)
{
    int e;

    erasehex(PSIZE, DSIZE, WSIZE);

    format = UNKNOWN;			/* auto-identify input style */

    if ( (e = loadhex(hfp, PSIZE, DSIZE, IBASE, CBASE, DBASE)) != 0 )
	telluser(pname,"While loading hexfile:",e);

    return 0;
}


#ifndef LIBVERSION

int main(int argc, char *argv[])
{
    FILE *hfp;

    if ( argc <= 1 )
	quit(pname,usage);

    if ( (hfp = fopen(argv[1],"r")) == NULL )
	quit(pname,"Can't open hexfile");

    whathex(hfp);	       /* read hexfile and thereby update info */

    printf("Format: %s\n",(format==INHX8M)? "INHX8M": "INHX16");

    if ( pmlast >= 0 )
	printf("Last program location: %04X\n",pmlast);
    else
	printf("No program words\n");

    if ( dmlast >= 0 )
	printf("Last data location: %04X\n",dmlast);
    else
	printf("No data words\n");

    if ( id )
	showid(stdout);
    else
	printf("No ID words\n");

    if ( cf )
	showcf(stdout);
    else
	printf("No config word\n");

    printf("Checksum: %04X (%04X)\n",check84(),checkall());

    if ( pmlast < DSIZE && dmlast < 0 && !id && !cf )
	printf("(Could be a data file with origin at 0)\n");

    return EXIT_SUCCESS;
}

#endif  /* LIBVERSION */